@@ -21,6 +21,9 @@
 
 #include <drivers/atmodem/atutil.h>
 
+extern void huawei_ussd_init(void);
+extern void huawei_ussd_exit(void);
+
 extern void huawei_voicecall_init(void);
 extern void huawei_voicecall_exit(void);
 
