@@ -31,6 +31,7 @@
 
 #include "ofono.h"
 #include "common.h"
+#include "hfp.h"
 #include "gatserver.h"
 #include "gatppp.h"
 
@@ -100,9 +101,6 @@ static void cleanup_ppp(struct ofono_emulator *em)
 	__ofono_private_network_release(em->pns_id);
 	em->pns_id = 0;
 
-	if (em->server == NULL)
-		return;
-
 	g_at_server_resume(em->server);
 	g_at_server_send_final(em->server, G_AT_SERVER_RESULT_NO_CARRIER);
 }
@@ -421,13 +419,6 @@ static gboolean notify_ring(void *user_data)
 
 	c = find_call_with_status(em, CALL_STATUS_INCOMING);
 
-	/*
-	 * In case of waiting call becoming an incoming call, call status
-	 * change may not have been done yet, so try to find waiting call too
-	 */
-	if (c == NULL)
-		c = find_call_with_status(em, CALL_STATUS_WAITING);
-
 	if (c == NULL)
 		return TRUE;
 
@@ -462,7 +453,7 @@ static void brsf_cb(GAtServer *server, GAtServerRequestType type,
 		if (g_at_result_iter_next_number(&iter, &val) == FALSE)
 			goto fail;
 
-		if (val < 0 || val > 127)
+		if (val < 0 || val > 255)
 			goto fail;
 
 		em->r_features = val;
@@ -1252,3 +1243,26 @@ start_ring:
 	em->callsetup_source = g_timeout_add_seconds(RING_TIMEOUT,
 							notify_ring, em);
 }
+
+void __ofono_emulator_set_indicator_forced(struct ofono_emulator *em,
+						const char *name, int value)
+{
+	int i;
+	struct indicator *ind;
+	char buf[20];
+
+	ind = find_indicator(em, name, &i);
+
+	if (ind == NULL || value < ind->min || value > ind->max)
+		return;
+
+	ind->value = value;
+
+	if (em->events_mode == 3 && em->events_ind && em->slc && ind->active) {
+		if (!g_at_server_command_pending(em->server)) {
+			sprintf(buf, "+CIEV: %d,%d", i, ind->value);
+			g_at_server_send_unsolicited(em->server, buf);
+		} else
+			ind->deferred = TRUE;
+	}
+}
