@@ -2355,6 +2355,46 @@ error:
 		g_free(v);
 }
 
+void ofono_voicecall_mpty_hint(struct ofono_voicecall *vc, unsigned int ids)
+{
+	GSList *old;
+	GSList *l;
+
+	DBG("ids: %u", ids);
+
+	/* id of 0 is never valid for a call */
+	if (ids & 0x1)
+		return;
+
+	/* Ignore the hint if there's nothing to do */
+	if (__builtin_popcount(ids) < 2 && vc->multiparty_list == NULL)
+		return;
+
+	old = vc->multiparty_list;
+	vc->multiparty_list = NULL;
+
+	for (l = vc->call_list; l; l = l->next) {
+		struct voicecall *v = l->data;
+
+		if (ids & (1 << v->call->id))
+			vc->multiparty_list =
+				g_slist_prepend(vc->multiparty_list, v);
+	}
+
+	if (vc->multiparty_list)
+		vc->multiparty_list = g_slist_reverse(vc->multiparty_list);
+
+	if (g_slist_length(vc->multiparty_list) == 1) {
+		ofono_error("Created multiparty list length is 1"
+				", which would indicate a bug in the driver"
+				" or the remote device");
+		vc->multiparty_list = NULL;
+	}
+
+	voicecalls_multiparty_changed(old, vc->multiparty_list);
+	g_slist_free(old);
+}
+
 static void send_ciev_after_swap_callback(const struct ofono_error *error,
 								void *data)
 {
@@ -2834,6 +2874,7 @@ static void sim_state_watch(enum ofono_sim_state new_state, void *user)
 						read_sim_ecc_numbers, vc, NULL);
 		break;
 	case OFONO_SIM_STATE_NOT_PRESENT:
+	case OFONO_SIM_STATE_RESETTING:
 		/* TODO: Must release all non-emergency calls */
 
 		if (vc->sim_context) {
@@ -3675,7 +3716,7 @@ int __ofono_voicecall_dial(struct ofono_voicecall *vc,
 {
 	struct dial_request *req;
 
-	if (!valid_phone_number_format(addr))
+	if (!valid_long_phone_number_format(addr))
 		return -EINVAL;
 
 	if (vc->driver->dial == NULL)
@@ -3868,15 +3909,21 @@ void __ofono_voicecall_set_alpha_and_icon_id(struct ofono_voicecall *vc,
 	vc->dial_req = req;
 
 	vc->flags |= VOICECALL_FLAG_STK_MODEM_CALLSETUP;
+
+	DBG("%p, %p", vc, vc->dial_req);
 }
 
 void __ofono_voicecall_clear_alpha_and_icon_id(struct ofono_voicecall *vc)
 {
-	g_free(vc->dial_req->message);
-	vc->dial_req->message = NULL;
+	DBG("%p, %p", vc, vc->dial_req);
 
-	g_free(vc->dial_req);
-	vc->dial_req = NULL;
+	if (vc->dial_req) {
+		g_free(vc->dial_req->message);
+		vc->dial_req->message = NULL;
+
+		g_free(vc->dial_req);
+		vc->dial_req = NULL;
+	}
 
 	vc->flags &= ~VOICECALL_FLAG_STK_MODEM_CALLSETUP;
 }
